@@ -44,7 +44,6 @@ create_main (void)
     GdkPixbuf *main_icon_pixbuf;
     GtkWidget *vbox1;
     GtkWidget *toolbar1;
-    gint tmp_toolbar_icon_size;
     GtkWidget *lookup;
     GtkWidget *preferences;
     GtkWidget *separatortoolitem1;
@@ -78,7 +77,7 @@ create_main (void)
     if (main_icon_pixbuf)
     {
         gtk_window_set_icon (GTK_WINDOW (main_win), main_icon_pixbuf);
-        gdk_pixbuf_unref (main_icon_pixbuf);
+        g_object_unref (main_icon_pixbuf);
     }
 
     vbox1 = gtk_vbox_new (FALSE, 0);
@@ -89,7 +88,6 @@ create_main (void)
     gtk_widget_show (toolbar1);
     gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
     gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_BOTH_HORIZ);
-    tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1));
     
     GtkWidget* icon;
     icon = gtk_image_new_from_stock(GTK_STOCK_REFRESH, gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar1)));
@@ -650,7 +648,7 @@ create_prefs (void)
                                         G_CALLBACK (on_rip_ogg_toggled),
                                         NULL);
     /* END OGG */
-    
+
     /* FLAC */
     frame5 = gtk_frame_new (NULL);
     gtk_widget_show (frame5);
@@ -706,6 +704,59 @@ create_prefs (void)
     gtk_widget_show (hiddenbox);
     gtk_container_add (GTK_CONTAINER (expander), hiddenbox);
     
+    /* OPUS */
+    GtkWidget *opusLbl;
+    GtkWidget *opusrate;
+    GtkWidget *rip_opus;
+    GtkWidget *opus_frame;
+    GtkWidget *opushbox;
+    GtkWidget *opusalignment;
+    char opus_kbps[10];
+
+    opus_frame = gtk_frame_new(NULL);
+    gtk_widget_show (opus_frame);
+    gtk_box_pack_start (GTK_BOX(hiddenbox), opus_frame, FALSE, FALSE, 0);
+
+    opusalignment = gtk_alignment_new (0.5, 0.5, 1, 1);
+    gtk_widget_show(opusalignment);
+    gtk_container_add (GTK_CONTAINER(opus_frame),opusalignment);
+    gtk_alignment_set_padding (GTK_ALIGNMENT(opusalignment), 2, 2, 12, 2);
+
+    opushbox = gtk_hbox_new(FALSE,0);
+    gtk_widget_show (opushbox);
+    gtk_container_add (GTK_CONTAINER(opusalignment), opushbox);
+
+    opusLbl = gtk_label_new (_("Bitrate"));
+    gtk_widget_show(opusLbl);
+    gtk_box_pack_start (GTK_BOX(opushbox), opusLbl, FALSE, FALSE,0);
+    GLADE_HOOKUP_OBJECT (prefs, opusLbl, "opus_lbl");
+
+    opusrate = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 13, 1, 1, 1)));
+    gtk_widget_show(opusrate);
+    gtk_box_pack_start(GTK_BOX(opushbox), opusrate, TRUE, TRUE, 5);
+    gtk_scale_set_draw_value (GTK_SCALE (opusrate), FALSE);
+    gtk_scale_set_digits (GTK_SCALE (opusrate), 0);
+    g_signal_connect ((gpointer) opusrate, "value_changed",
+                                        G_CALLBACK (on_opusrate_value_changed),
+                                        NULL);
+    tooltips = gtk_tooltips_new ();
+    gtk_tooltips_set_tip (tooltips, opusrate, _("Higher bitrate is better quality but also bigger file. Most people use 160Kbps."), NULL);
+    GLADE_HOOKUP_OBJECT (prefs, opusrate, "opusrate");
+    snprintf(opus_kbps, 10, _("%dKbps"), 32);
+    label = gtk_label_new (kbps_text);
+    gtk_widget_show (label);
+    gtk_box_pack_start (GTK_BOX (opushbox), label, FALSE, FALSE, 0);
+    GLADE_HOOKUP_OBJECT (prefs, label, "bitrate_lbl_4");
+
+    rip_opus = gtk_check_button_new_with_mnemonic (_("OPUS (lossy compression)"));
+    gtk_widget_show (rip_opus);
+    gtk_frame_set_label_widget(GTK_FRAME(opus_frame), rip_opus);
+    g_signal_connect((gpointer) rip_opus, "toggled",
+                     G_CALLBACK (on_rip_opus_toggled),
+                     NULL);
+    GLADE_HOOKUP_OBJECT (prefs, rip_opus, "rip_opus");
+    /* END OPUS */
+
     /* WAVPACK */
     GtkWidget* flacVbox;
     
@@ -1273,6 +1324,20 @@ void enable_ogg_widgets(void)
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "oggquality"), TRUE);
 }
 
+void disable_opus_widgets(void)
+{
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "opus_lbl"),FALSE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs,"opusrate"), FALSE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "bitrate_lbl_4"), FALSE);
+}
+
+void enable_opus_widgets(void)
+{
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "opus_lbl"),TRUE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs,"opusrate"), TRUE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "bitrate_lbl_4"), TRUE);
+}
+
 void disable_flac_widgets(void)
 {
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "flac_lbl"), FALSE);
@@ -1349,7 +1414,7 @@ void enable_musepack_widgets(void)
 
 #if GTK_MINOR_VERSION >= 6
 static const char* 
-GBLprogramName = "Asunder 2.2";
+GBLprogramName = "Asunder 2.4";
 
 static const char* 
 GBLauthors[2] = {
@@ -1384,11 +1449,27 @@ GBLauthors[2] = {
 "- A kFreeBSD fix.\n"
 "\n"
 "Richard Gill\n"
-"- Support for XDG_CONFIG_HOME.\n"
+"- Support for XDG_CONFIG_HOME\n"
+"- Support for XDG_CACHE_HOME\n"
 "\n"
 "Hiroyuki Ito\n"
 "- Support for autocompletion in artist/album/genre fields.\n"
 "\n"
+"Jonathan 'theJPster' Pallant\n"
+"- Tag AAC files using neroAacTag.\n"
+"\n"
+"Micah Lindstrom\n"
+"- Support for invalid filesystem characters in metadata.\n"
+"\n"
+"Tim Allen\n"
+"- Fix to make the MusicBrainz CDDB gateway work with Asunder.\n"
+"\n"
+"Tudor\n"
+"- Desktop file trick to add Asunder to the list of audio CD apps in Gnome.\n"
+"\n"
+"Felix Braun\n"
+"- Support for encoding into OPUS.\n"
+"\n"
 "Packages:\n"
 "\n"
 "Toni Graffy\n"
@@ -1458,7 +1539,10 @@ GBLtranslators =
 "- be (Belarusian) translation of Asunder versions 1.0.2 - 1.5\n"
 "\n"
 "Emil Krumov\n"
-"- bg (Bulgarian) translation of Asunder versions 1.0.2 - 1.9\n"
+"- bg (Bulgarian) translation of Asunder versions 1.0.2 - 2.2\n"
+"\n"
+"Rajib Lochan Dhibar\n"
+"- bn_IN (Bengali / India) translation of Asunder version 2.3\n"
 "\n"
 "Davor Buday\n"
 "- bs_BA (Bosnian / Bosnia and Herzegovina) translation of Asunder version 2.1\n"
@@ -1469,6 +1553,9 @@ GBLtranslators =
 "Schmaki\n"
 "- cs (Czeck) translation of Asunder versions 0.8 - 1.0, 1.9\n"
 "\n"
+"Petr Simacek\n"
+"- cs (Czeck) translation of Asunder version 2.2\n"
+"\n"
 "Joe Hansen\n"
 "- da (Danish) translation of Asunder version 1.9\n"
 "\n"
@@ -1493,18 +1580,30 @@ GBLtranslators =
 "Marco Antonio\n"
 "- es (Spanish) translation of Asunder version 1.9.3\n"
 "\n"
+"Marco Antonio Frias B\n"
+"- es (Spanish) translation of Asunder version 2.2\n"
+"\n"
 "Guillermo Gras\n"
 "- eu (Basque) translation of Asunder version 2.0\n"
 "\n"
 "Eero Salokannel\n"
-"- fi (Finnish) translation of Asunder versions 0.8 - 1.5\n"
+"- fi (Finnish) translation of Asunder versions 0.8 - 2.2\n"
 "\n"
 "Christophe Legras\n"
 "- fr (French) translation of Asunder versions 0.8 - 1.9\n"
 "\n"
 "Fabrice Le Goff\n"
 "- fr (French) translation of Asunder version 1.6\n"
-"\n"         
+"\n"
+"Clement Raievsky\n"
+"- fr (French) translation of Asunder versions 2.1 - 2.2\n"
+"\n"
+"Miguel Anxo Bouzada\n"
+" - gl (Galician) translation of Asunder version 2.2\n"
+"\n"
+"Genghis Khan\n"
+" - he (Hebrew) translation of Asunder version 2.2\n"
+"\n"
 "Peter Polonkai\n"
 "- hu (Hungarian) translation of Asunder versions 0.8 - 1.9\n"
 "\n"      
@@ -1512,7 +1611,7 @@ GBLtranslators =
 "- hr (Croatian) translation of Asunder version 1.9\n"
 "\n"
 "Pader Rezso\n"
-"- hu (Hungarian) translation of Asunder version 1.9.2\n"
+"- hu (Hungarian) translation of Asunder versions 1.9.2 - 2.3\n"
 "\n"
 "Valerio Guaglianone\n"
 "- it (Italian) translation of Asunder versions 0.8 - 0.8.1, 1.6 - 1.9\n"
@@ -1520,6 +1619,9 @@ GBLtranslators =
 "Fabio Vergnani\n"
 "- it (Italian) translation of Asunder version 1.0\n"
 "\n"
+"Fabio Boccaletti\n"
+"- it (Italian) translation of Asunder version 2.3\n"
+"\n"
 "UTUMI Hirosi\n"
 "- ja (Japanese) translation of Asunder versions 1.0 - 1.5\n"
 "\n"
@@ -1562,6 +1664,12 @@ GBLtranslators =
 "Pedro Henrique\n"
 "- pt_BR (Portuguese/Brazil) translation of Asunder version 2.1\n"
 "\n"
+"Neliton Pereira Jr\n"
+"- pt_BR (Portuguese/Brazil) translation of Asunder version 2.2\n"
+"\n"
+"Paulo Roberto de Oliveira Castro\n"
+"- pt_BR (Portuguese/Brazil) translation of Asunder version 2.2\n"
+"\n"
 "Alexandre Prokoudine\n"
 "- ru (Russian) translation of Asunder version 1.9.2\n"
 "\n"
@@ -1589,6 +1697,9 @@ GBLtranslators =
 "Daniel Nylander\n"
 "- sv (Swedish) translation of Asunder versions 0.8 - 1.9\n"
 "\n"
+"Andreas Ronnquist\n"
+"- sv (Swedish) translation of Asunder version 2.2\n"
+"\n"
 "Savas Sen\n"
 "- tr (Turkish) translation of Asunder version 1.9\n"
 "\n"
@@ -1596,6 +1707,9 @@ GBLtranslators =
 "http://makki.urducoder.com/\n"
 "- ur_PK (Urdu/Pakistan) translation of Asunder version 2.0\n"
 "\n"
+"Tiansworld\n"
+"- zh_CN (Simplified Chineese) translation of Asunder version 2.3\n"
+"\n"
 "Cheng-Wei Chien\n"
 "- zh_TW (Chineese/Taiwan) translation of Asunder versions 0.8 - 1.9\n"
 "\n";
@@ -1603,7 +1717,7 @@ GBLtranslators =
 static const char* 
 GBLcomments = 
 N_("An application to save tracks from an Audio CD \n"
-"as WAV, MP3, OGG, FLAC, Wavpack, Musepack, Monkey's Audio, and/or "
+"as WAV, MP3, OGG, FLAC, Opus, Wavpack, Musepack, Monkey's Audio, and/or "
 "AAC files.");
 
 static const char* 
