@@ -63,7 +63,6 @@ int gbl_null_fd;
 
 int main(int argc, char *argv[])
 {
-    GtkTreeViewColumn *col;
     GtkCellRenderer *renderer;
     
 #ifdef ENABLE_NLS
@@ -117,33 +116,28 @@ int main(int argc, char *argv[])
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Rip"), renderer, "active", COL_RIPTRACK, NULL);
 
-    col = gtk_tree_view_column_new();
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Track"), renderer, "text", COL_TRACKNUM, NULL);
 
-    col = gtk_tree_view_column_new();
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "editable", TRUE, NULL);
     g_signal_connect(renderer, "edited", (GCallback) on_artist_edited, NULL);
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Artist"), renderer, "text", COL_TRACKARTIST, NULL);
 
-    col = gtk_tree_view_column_new();
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "editable", TRUE, NULL);
     g_signal_connect(renderer, "edited", (GCallback) on_title_edited, NULL);
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Title"), renderer, "text", COL_TRACKTITLE, NULL);
 
-    col = gtk_tree_view_column_new();						// lnr
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "editable", TRUE, NULL);
     g_signal_connect(renderer, "edited", (GCallback) on_genre_edited, NULL);
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Genre"), renderer, "text", COL_GENRE, NULL);
 
-    col = gtk_tree_view_column_new();
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Time"), renderer, "text", COL_TRACKTIME, NULL);
@@ -316,11 +310,11 @@ GtkTreeModel * create_model_from_disc(cddb_disc_t * disc)
         snprintf(time, 6, "%02d:%02d", seconds/60, seconds%60);
         
         track_artist = (char*)cddb_track_get_artist(track);
-        trim_chars(track_artist, BADCHARS);		// lnr
+        //trim_chars(track_artist, BADCHARS);		// lnr	//Commented out by mrpl
         trim_whitespace(track_artist);
         
         track_title = (char*)cddb_track_get_title(track); //!! this returns const char*
-        trim_chars(track_title, BADCHARS);		// lnr
+        //trim_chars(track_title, BADCHARS);		// lnr	//Commented out by mrpl
         trim_whitespace(track_title);
         
         char year_str[5];
@@ -342,10 +336,13 @@ GtkTreeModel * create_model_from_disc(cddb_disc_t * disc)
     return GTK_TREE_MODEL(store);
 }
 
-
+// The original code only worked on Linux, commented it out to work 
+// on BSD as well.
+// The new code will hopefully work everywhere, but left the old two
+// here for posterity's reference.
 void eject_disc(char * cdrom)
 {
-    int fd;
+    /*int fd;
     
     // open the device
     fd = open(cdrom, O_RDONLY | O_NONBLOCK);
@@ -367,7 +364,18 @@ void eject_disc(char * cdrom)
 #endif
     //~ }
     
-    close(fd);
+    close(fd);*/
+    
+    if (!fork())
+    {
+        // Wonderful const magic to make compiler happy
+        const char* const args[] = {"eject", cdrom, NULL};
+        execvp(args[0], (char*const*)args);
+        
+        printf("Should never see this, why did the call to 'eject' fail?\n");
+        debugLog("Should never see this, why did the call to 'eject' fail?\n");
+        exit(1);
+    }
 }
 
 static GThread * gbl_cddb_query_thread;
@@ -375,13 +383,35 @@ static int gbl_cddb_query_thread_running;
 static cddb_conn_t * gbl_cddb_query_thread_conn;
 static cddb_disc_t * gbl_cddb_query_thread_disc;
 static int gbl_cddb_query_thread_num_matches;
+static GList * gbl_matches = NULL;
 
 gpointer cddb_query_thread_run(gpointer data)
 {
+    int i;
+    
     gbl_cddb_query_thread_num_matches = cddb_query(gbl_cddb_query_thread_conn, gbl_cddb_query_thread_disc);
     if(gbl_cddb_query_thread_num_matches == -1)
         gbl_cddb_query_thread_num_matches = 0;
     
+    // make a list of all the matches
+    for (i = 0; i < gbl_cddb_query_thread_num_matches; i++)
+    {
+        cddb_disc_t * possible_match = cddb_disc_clone(gbl_cddb_query_thread_disc);
+        if (cddb_read(gbl_cddb_query_thread_conn, possible_match) == 1)
+        {
+            gbl_matches = g_list_append(gbl_matches, possible_match);
+            
+            // move to next match
+            if (i < gbl_cddb_query_thread_num_matches - 1)
+            {
+                if (!cddb_query_next(gbl_cddb_query_thread_conn, gbl_cddb_query_thread_disc))
+                    fatalError("Query index out of bounds.");
+            }
+        }
+        else
+            printf("Failed to cddb_read()\n");
+    }
+    
     g_atomic_int_set(&gbl_cddb_query_thread_running, 0);
     
     return NULL;
@@ -389,9 +419,6 @@ gpointer cddb_query_thread_run(gpointer data)
 
 GList * lookup_disc(cddb_disc_t * disc)
 {
-    int i;
-    GList * matches = NULL;
-    
     // set up the connection to the cddb server
     gbl_cddb_query_thread_conn = cddb_new();
     if (gbl_cddb_query_thread_conn == NULL)
@@ -407,6 +434,10 @@ GList * lookup_disc(cddb_disc_t * disc)
         cddb_http_proxy_enable(gbl_cddb_query_thread_conn);
     }
     
+    // force HTTP when port 80 (for MusicBrainz). This code by Tim.
+    if (global_prefs->cddb_port_number == 80)
+        cddb_http_enable(gbl_cddb_query_thread_conn);
+    
     // query cddb to find similar discs
     g_atomic_int_set(&gbl_cddb_query_thread_running, 1);
     gbl_cddb_query_thread_disc = disc;
@@ -432,28 +463,9 @@ GList * lookup_disc(cddb_disc_t * disc)
         enable_all_main_widgets();
     gdk_threads_leave();
     
-    // make a list of all the matches
-    for (i = 0; i < gbl_cddb_query_thread_num_matches; i++)
-    {
-        cddb_disc_t * possible_match = cddb_disc_clone(disc);
-        if (!cddb_read(gbl_cddb_query_thread_conn, possible_match))
-        {
-            cddb_error_print(cddb_errno(gbl_cddb_query_thread_conn));
-            fatalError("cddb_read() failed.");
-        }
-        matches = g_list_append(matches, possible_match);
-        
-        // move to next match
-        if (i < gbl_cddb_query_thread_num_matches - 1)
-        {
-            if (!cddb_query_next(gbl_cddb_query_thread_conn, disc))
-                fatalError("Query index out of bounds.");
-        }
-    }
-    
     cddb_destroy(gbl_cddb_query_thread_conn);
     
-    return matches;
+    return gbl_matches;
 }
 
 
@@ -604,7 +616,7 @@ void update_tracklist(cddb_disc_t * disc)
     
     if (disc_artist != NULL)
     {
-        trim_chars(disc_artist, BADCHARS);			// lnr
+        //trim_chars(disc_artist, BADCHARS);			// lnr	//Commented out by mrpl
         trim_whitespace(disc_artist);
         gtk_entry_set_text(GTK_ENTRY(album_artist), disc_artist);
         
@@ -621,14 +633,14 @@ void update_tracklist(cddb_disc_t * disc)
     }
     if (disc_title != NULL)
     {
-        trim_chars(disc_title, BADCHARS);			// lnr
+        //trim_chars(disc_title, BADCHARS);			// lnr	//Commented out by mrpl
         trim_whitespace(disc_title);
         gtk_entry_set_text(GTK_ENTRY(album_title), disc_title);
     }
     
     if ( disc_genre )								// lnr
     {
-        trim_chars( disc_genre, BADCHARS);
+        //trim_chars( disc_genre, BADCHARS);			//Commented out by mrpl
         trim_whitespace( disc_genre );
 
         gtk_entry_set_text( GTK_ENTRY( album_genre ), disc_genre );
