@@ -36,6 +36,7 @@ static int counter;
 static FILE * playlist_wav = NULL;
 static FILE * playlist_mp3 = NULL;
 static FILE * playlist_ogg = NULL;
+static FILE * playlist_opus = NULL;
 static FILE * playlist_flac = NULL;
 static FILE * playlist_wavpack = NULL;
 static FILE * playlist_monkey = NULL;
@@ -57,6 +58,7 @@ static int tracks_to_rip;
 static double rip_percent;
 static double mp3_percent;
 static double ogg_percent;
+static double opus_percent;
 static double flac_percent;
 static double wavpack_percent;
 static double monkey_percent;
@@ -79,6 +81,8 @@ void abort_threads()
         kill(lame_pid, SIGKILL);
     if (oggenc_pid != 0) 
         kill(oggenc_pid, SIGKILL);
+    if (opusenc_pid !=0)
+        kill(opusenc_pid, SIGKILL);
     if (flac_pid != 0) 
         kill(flac_pid, SIGKILL);
     if (wavpack_pid != 0) 
@@ -92,7 +96,7 @@ void abort_threads()
     
     /* wait until all the worker threads are done */
     while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || 
-           flac_pid != 0 || wavpack_pid != 0 || monkey_pid != 0 || 
+           opusenc_pid != 0 || flac_pid != 0 || wavpack_pid != 0 || monkey_pid != 0 ||
            musepack_pid != 0 || aac_pid != 0)
     {
         debugLog("w1");
@@ -115,8 +119,8 @@ void abort_threads()
     gdk_flush();
     working = false;
     
-    show_completed_dialog(numCdparanoiaOk + numLameOk + numOggOk + numFlacOk + numWavpackOk + numMonkeyOk + numMusepackOk + numAacOk,
-                          numCdparanoiaFailed + numLameFailed + numOggFailed + numFlacFailed + numWavpackFailed + numMonkeyFailed + numMusepackFailed + numAacFailed);
+    show_completed_dialog(numCdparanoiaOk + numLameOk + numOggOk + numOpusOk + numFlacOk + numWavpackOk + numMonkeyOk + numMusepackOk + numAacOk,
+                          numCdparanoiaFailed + numLameFailed + numOggFailed + numOpusFailed + numFlacFailed + numWavpackFailed + numMonkeyFailed + numMusepackFailed + numAacFailed);
 }
 
 // spawn needed threads and begin ripping
@@ -131,6 +135,7 @@ void dorip()
     rip_percent = 0.0;
     mp3_percent = 0.0;
     ogg_percent = 0.0;
+    opus_percent = 0.0;
     flac_percent = 0.0;
     wavpack_percent = 0.0;
     monkey_percent = 0.0;
@@ -147,14 +152,25 @@ void dorip()
     const char * albumtitle = gtk_entry_get_text(GTK_ENTRY(lookup_widget(win_main, "album_title")));
     const char * albumyear = gtk_entry_get_text(GTK_ENTRY(lookup_widget(win_main, "album_year")));
     const char * albumgenre = gtk_entry_get_text(GTK_ENTRY(lookup_widget(win_main, "album_genre")));
-    char * albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist, albumtitle, albumgenre, NULL);
-    char * playlist = parse_format(global_prefs->format_playlist, 0, albumyear, albumartist, albumtitle, albumgenre, NULL);
+    
+    //Trimmed for use in filenames	//mrpl
+    char * albumartist_trimmed = strdup(albumartist);
+    trim_chars(albumartist_trimmed, BADCHARS);
+    char * albumtitle_trimmed = strdup(albumtitle);
+    trim_chars(albumtitle_trimmed, BADCHARS);
+    char * albumgenre_trimmed = strdup(albumgenre);
+    trim_chars(albumgenre_trimmed, BADCHARS);
+
+    //char * albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist, albumtitle, albumgenre, NULL);
+    //char * playlist = parse_format(global_prefs->format_playlist, 0, albumyear, albumartist, albumtitle, albumgenre, NULL);
+    char * albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, NULL);
+    char * playlist = parse_format(global_prefs->format_playlist, 0, albumyear, albumartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, NULL);
     
     overwriteAll = false;
     overwriteNone = false;
     
     // make sure there's at least one format to rip to
-    if (!global_prefs->rip_wav && !global_prefs->rip_mp3 && !global_prefs->rip_ogg && 
+    if (!global_prefs->rip_wav && !global_prefs->rip_mp3 && !global_prefs->rip_ogg && !global_prefs->rip_opus &&
         !global_prefs->rip_flac && !global_prefs->rip_wavpack && !global_prefs->rip_monkey &&
         !global_prefs->rip_musepack && !global_prefs->rip_aac)
     {
@@ -254,6 +270,14 @@ void dorip()
             
             free(filename);
         }
+        if (global_prefs->rip_opus)
+        {
+            char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "opus.m3u");
+            
+            make_playlist(filename, &playlist_opus);
+            
+            free(filename);
+        }
         if (global_prefs->rip_flac)
         {
             char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "flac.m3u");
@@ -298,12 +322,16 @@ void dorip()
     
     free(albumdir);
     free(playlist);
+    free(albumartist_trimmed);
+    free(albumtitle_trimmed);
+    free(albumgenre_trimmed);
     
     gtk_widget_show(win_ripping);
     
     numCdparanoiaFailed = 0;
     numLameFailed = 0;
     numOggFailed = 0;
+    numOpusFailed = 0;
     numFlacFailed = 0;
     numWavpackFailed = 0;
     numMonkeyFailed = 0;
@@ -313,6 +341,7 @@ void dorip()
     numCdparanoiaOk = 0;
     numLameOk = 0;
     numOggOk = 0;
+    numOpusOk = 0;
     numFlacOk = 0;
     numWavpackOk = 0;
     numMonkeyOk = 0;
@@ -333,6 +362,8 @@ gpointer rip(gpointer data)
     int tracknum;
     const char * trackartist;
     const char * tracktitle;
+    char * trackartist_trimmed;
+    char * tracktitle_trimmed;
     
     char * albumdir = NULL;
     char * musicfilename = NULL;
@@ -348,6 +379,15 @@ gpointer rip(gpointer data)
         
         gboolean rowsleft = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
     gdk_threads_leave();
+    
+    //Trimmed for use in filenames	//mrpl
+    char * albumartist_trimmed = strdup(albumartist);
+    trim_chars(albumartist_trimmed, BADCHARS);
+    char * albumtitle_trimmed = strdup(albumtitle);
+    trim_chars(albumtitle_trimmed, BADCHARS);
+    char * albumgenre_trimmed = strdup(albumgenre);
+    trim_chars(albumgenre_trimmed, BADCHARS);
+    
     while(rowsleft)
     {
         const char * trackyear = 0;
@@ -368,9 +408,17 @@ gpointer rip(gpointer data)
         
         if (riptrack)
         {
-            albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist, albumtitle, albumgenre, NULL);
-            //~ musicfilename = parse_format(global_prefs->format_music, tracknum, trackyear, trackartist, albumtitle, tracktitle);
-            musicfilename = parse_format(global_prefs->format_music, tracknum, albumyear, trackartist, albumtitle, albumgenre, tracktitle);
+            //Trimmed for use in filenames	//mrpl
+            trackartist_trimmed = strdup(trackartist);
+            trim_chars(trackartist_trimmed, BADCHARS);
+            tracktitle_trimmed = strdup(tracktitle);
+            trim_chars(tracktitle_trimmed, BADCHARS);
+            
+            //~albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist, albumtitle, albumgenre, NULL);
+            //~musicfilename = parse_format(global_prefs->format_music, tracknum, trackyear, trackartist, albumtitle, tracktitle);
+            //~musicfilename = parse_format(global_prefs->format_music, tracknum, albumyear, trackartist, albumtitle, albumgenre, tracktitle);            
+            albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, NULL);
+            musicfilename = parse_format(global_prefs->format_music, tracknum, albumyear, trackartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, tracktitle_trimmed);
             wavfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wav");
             
             debugLog("Ripping track %d to \"%s\"\n", tracknum, wavfilename);
@@ -396,10 +444,12 @@ gpointer rip(gpointer data)
             
             if(doRip)
                 cdparanoia(global_prefs->cdrom, tracknum, wavfilename, &rip_percent);
-
+            
             free(albumdir);
             free(musicfilename);
             free(wavfilename);
+            free(trackartist_trimmed);
+            free(tracktitle_trimmed);
             
             rip_percent = 0.0;
             rip_tracks_completed++;
@@ -418,6 +468,10 @@ gpointer rip(gpointer data)
         gdk_threads_leave();
     }
     
+    free(albumartist_trimmed);
+    free(albumtitle_trimmed);
+    free(albumgenre_trimmed);
+    
     // no more tracks to rip, safe to eject
     if (global_prefs->eject_on_done)
     {
@@ -448,11 +502,19 @@ gpointer encode(gpointer data)
     char* album_genre = NULL;		// lnr
     char* album_year = NULL;
     
+    char* album_artist_trimmed = NULL;
+    char* album_title_trimmed = NULL;
+    char* album_genre_trimmed = NULL;
+    char* trackartist_trimmed = NULL;
+    char* tracktitle_trimmed = NULL;
+    char* genre_trimmed = NULL;
+    
     char* albumdir = NULL;
     char* musicfilename = NULL;
     char* wavfilename = NULL;
     char* mp3filename = NULL;
     char* oggfilename = NULL;
+    char* opusfilename = NULL;
     char* flacfilename = NULL;
     char* wavpackfilename = NULL;
     char* wavpackfilename2 = NULL;
@@ -499,10 +561,18 @@ gpointer encode(gpointer data)
         strcpy( album_genre, temp_album_genre );
         add_completion(album_genre_widget);
         save_completion(album_genre_widget);
-
+        
         gboolean rowsleft = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
     gdk_threads_leave();
-
+    
+    //Trimmed for use in filenames	//mrpl
+    album_artist_trimmed = strdup(album_artist);
+    trim_chars(album_artist_trimmed, BADCHARS);
+    album_title_trimmed = strdup(album_title);
+    trim_chars(album_title_trimmed, BADCHARS);
+    album_genre_trimmed = strdup(album_genre);
+    trim_chars(album_genre_trimmed, BADCHARS);
+    
     while(rowsleft)
     {
         g_mutex_lock(barrier);
@@ -537,12 +607,24 @@ gpointer encode(gpointer data)
         
         if (riptrack)
         {
-            albumdir = parse_format(global_prefs->format_albumdir, 0, album_year, album_artist, album_title, genre, NULL);
-            //~ musicfilename = parse_format(global_prefs->format_music, tracknum, trackyear, trackartist, album_title, tracktitle);
-            musicfilename = parse_format(global_prefs->format_music, tracknum, album_year, trackartist, album_title, genre, tracktitle);
+            //Trimmed for use in filenames	//mrpl
+            trackartist_trimmed = strdup(trackartist);
+            trim_chars(trackartist_trimmed, BADCHARS);
+            tracktitle_trimmed = strdup(tracktitle);
+            trim_chars(tracktitle_trimmed, BADCHARS);
+            genre_trimmed = strdup(genre);
+            trim_chars(genre_trimmed, BADCHARS);
+            
+            //~albumdir = parse_format(global_prefs->format_albumdir, 0, album_year, album_artist, album_title, genre, NULL);
+            //~musicfilename = parse_format(global_prefs->format_music, tracknum, trackyear, trackartist, album_title, tracktitle);
+            //~musicfilename = parse_format(global_prefs->format_music, tracknum, album_year, trackartist, album_title, genre, tracktitle);
+            albumdir = parse_format(global_prefs->format_albumdir, 0, album_year, album_artist_trimmed, album_title_trimmed, genre_trimmed, NULL);
+            musicfilename = parse_format(global_prefs->format_music, tracknum, album_year, trackartist_trimmed, album_title_trimmed, genre_trimmed, tracktitle_trimmed);
+            
             wavfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wav");
             mp3filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "mp3");
             oggfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "ogg");
+            opusfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "opus");
             flacfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "flac");
             wavpackfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wv");
             wavpackfilename2 = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wvc");
@@ -618,6 +700,38 @@ gpointer encode(gpointer data)
                     fflush(playlist_ogg);
                 }
             }
+            if (global_prefs->rip_opus)
+            {
+                debugLog("Encoding track %d to \"%s\"\n", tracknum, opusfilename);
+
+                if (aborted) g_thread_exit(NULL);
+
+                rc = stat(opusfilename, &statStruct);
+                if(rc == 0)
+                {
+                    gdk_threads_enter();
+                        if(confirmOverwrite(opusfilename))
+                            doEncode = true;
+                        else
+                            doEncode = false;
+                    gdk_threads_leave();
+                }
+                else
+                    doEncode = true;
+
+                if(doEncode)
+                    opusenc(tracknum, trackartist, album_title, tracktitle, album_year, genre, wavfilename,
+                           opusfilename, global_prefs->opus_bitrate, &opus_percent);
+
+                if (aborted) g_thread_exit(NULL);
+
+                if (playlist_opus)
+                {
+                    fprintf(playlist_opus, "#EXTINF:%d,%s - %s\n", (min*60)+sec, trackartist, tracktitle);
+                    fprintf(playlist_opus, "%s\n", basename(opusfilename));
+                    fflush(playlist_opus);
+                }
+            }
             if (global_prefs->rip_flac)
             {
                 debugLog("Encoding track %d to \"%s\"\n", tracknum, flacfilename);
@@ -779,7 +893,8 @@ gpointer encode(gpointer data)
                 
                 if(doEncode)
                 {
-                    aac(wavfilename, aacfilename,
+                    aac(tracknum, trackartist, album_title, tracktitle, genre, album_year,
+                        wavfilename, aacfilename,
                         global_prefs->aac_quality, 
                         &aac_percent);
                 }
@@ -815,14 +930,19 @@ gpointer encode(gpointer data)
             free(wavfilename);
             free(mp3filename);
             free(oggfilename);
+            free(opusfilename);
             free(flacfilename);
             free(wavpackfilename);
             free(monkeyfilename);
             free(musepackfilename);
             free(aacfilename);
+            free(trackartist_trimmed);
+            free(tracktitle_trimmed);
+            free(genre_trimmed);
             
             mp3_percent = 0.0;
             ogg_percent = 0.0;
+            opus_percent = 0.0;
             flac_percent = 0.0;
             wavpack_percent = 0.0;
             monkey_percent = 0.0;
@@ -841,6 +961,10 @@ gpointer encode(gpointer data)
     free(album_artist);
     free(album_title);
     free(album_genre);
+    free(album_year);
+    free(album_artist_trimmed);
+    free(album_title_trimmed);
+    free(album_genre_trimmed);
     
     if (playlist_wav) fclose(playlist_wav);
     playlist_wav = NULL;
@@ -848,6 +972,8 @@ gpointer encode(gpointer data)
     playlist_mp3 = NULL;
     if (playlist_ogg) fclose(playlist_ogg);
     playlist_ogg = NULL;
+    if (playlist_opus) fclose (playlist_opus);
+    playlist_opus = NULL;
     if (playlist_flac) fclose(playlist_flac);
     playlist_flac = NULL;
     if (playlist_wavpack) fclose(playlist_wavpack);
@@ -866,7 +992,7 @@ gpointer encode(gpointer data)
     
     /* wait until all the worker threads are done */
     while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || 
-           flac_pid != 0 || wavpack_pid != 0 || monkey_pid != 0 ||
+           opusenc_pid != 0 || flac_pid != 0 || wavpack_pid != 0 || monkey_pid != 0 ||
            musepack_pid != 0 || aac_pid != 0)
     {
         debugLog("w2");
@@ -880,8 +1006,8 @@ gpointer encode(gpointer data)
         gtk_widget_hide(win_ripping);
         gdk_flush();
         
-        show_completed_dialog(numCdparanoiaOk + numLameOk + numOggOk + numFlacOk + numWavpackOk + numMonkeyOk + numMusepackOk + numAacOk,
-                              numCdparanoiaFailed + numLameFailed + numOggFailed + numFlacFailed + numWavpackFailed + numMonkeyFailed + numMusepackFailed + numAacFailed);
+        show_completed_dialog(numCdparanoiaOk + numLameOk + numOggOk + numOpusOk + numFlacOk + numWavpackOk + numMonkeyOk + numMusepackOk + numAacOk,
+                              numCdparanoiaFailed + numLameFailed + numOggFailed + numOpusFailed + numFlacFailed + numWavpackFailed + numMonkeyFailed + numMusepackFailed + numAacFailed);
     gdk_threads_leave();
     
     return NULL;
@@ -895,6 +1021,8 @@ gpointer track(gpointer data)
         parts++;
     if(global_prefs->rip_ogg) 
         parts++;
+    if(global_prefs->rip_opus)
+        parts++;
     if(global_prefs->rip_flac) 
         parts++;
     if(global_prefs->rip_wavpack) 
@@ -938,10 +1066,10 @@ gpointer track(gpointer data)
         if (aborted) g_thread_exit(NULL);
         
         debugLog("completed tracks %d, rip %.2lf%%; encoded tracks %d, "
-                 "mp3 %.2lf%% ogg %.2lf%% flac %.2lf%% wavpack %.2lf%% "
+                 "mp3 %.2lf%% ogg %.2lf%% opus %.2lf%% flac %.2lf%% wavpack %.2lf%% "
                  "monkey %.2lf%% musepack %.2lf%% aac %.2lf%%\n\n", 
                  rip_tracks_completed, rip_percent*100, encode_tracks_completed, 
-                 mp3_percent*100, ogg_percent*100, flac_percent*100, wavpack_percent*100, 
+                 mp3_percent*100, ogg_percent*100, opus_percent*100, flac_percent*100, wavpack_percent*100,
                  monkey_percent*100,musepack_percent*100,aac_percent*100);
         
         prip = (rip_tracks_completed+rip_percent) / tracks_to_rip;
@@ -954,7 +1082,7 @@ gpointer track(gpointer data)
         {
             pencode = ((double)encode_tracks_completed/(double)tracks_to_rip) + 
                        ((mp3_percent+ogg_percent+flac_percent+wavpack_percent+monkey_percent
-                         +musepack_percent+aac_percent) /
+                         +opus_percent+musepack_percent+aac_percent) /
                         (parts-1) / tracks_to_rip);
             snprintf(sencode, 13, "%d%% (%d/%d)", (int)(pencode*100),
                      (encode_tracks_completed < tracks_to_rip)
