@@ -10,6 +10,8 @@ Foundation; version 2 of the licence.
 
 */
 
+#define _GNU_SOURCE
+
 #include <sys/types.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
@@ -30,6 +32,7 @@ Foundation; version 2 of the licence.
 pid_t cdparanoia_pid;
 pid_t lame_pid;
 pid_t oggenc_pid;
+pid_t opusenc_pid;
 pid_t flac_pid;
 pid_t wavpack_pid;
 pid_t monkey_pid;
@@ -39,6 +42,7 @@ pid_t aac_pid;
 int numCdparanoiaFailed;
 int numLameFailed;
 int numOggFailed;
+int numOpusFailed;
 int numFlacFailed;
 int numWavpackFailed;
 int numMonkeyFailed;
@@ -48,6 +52,7 @@ int numAacFailed;
 int numCdparanoiaOk;
 int numLameOk;
 int numOggOk;
+int numOpusOk;
 int numFlacOk;
 int numWavpackOk;
 int numMonkeyOk;
@@ -85,6 +90,7 @@ void unblockSigChld(void)
 extern pid_t cdparanoia_pid;
 extern pid_t lame_pid;
 extern pid_t oggenc_pid;
+extern pid_t opusenc_pid;
 extern pid_t flac_pid;
 extern pid_t wavpack_pid;
 extern pid_t monkey_pid;
@@ -103,9 +109,9 @@ void sigchld(int signum)
     
     pid = wait(&status);
     
-    debugLog("sigchld for %d (know about wav %d, mp3 %d, ogg %d, flac %d, "
+    debugLog("sigchld for %d (know about wav %d, mp3 %d, ogg %d, opus, %d, flac %d, "
              "wv %d, ape %d, mpc %d, m4a %d\n",
-             pid, cdparanoia_pid, lame_pid, oggenc_pid, flac_pid, 
+             pid, cdparanoia_pid, lame_pid, oggenc_pid, opusenc_pid, flac_pid,
              wavpack_pid, monkey_pid, musepack_pid, aac_pid);
     
     /* this is because i can't seem to be able to block sigchld: */
@@ -115,7 +121,7 @@ void sigchld(int signum)
         usleep(100);
     }
     
-    if (pid != cdparanoia_pid && pid != lame_pid && pid != oggenc_pid && 
+    if (pid != cdparanoia_pid && pid != lame_pid && pid != oggenc_pid && pid != opusenc_pid &&
         pid != flac_pid && pid != wavpack_pid && pid != monkey_pid &&
         pid != musepack_pid && pid != aac_pid)
     {
@@ -150,6 +156,11 @@ void sigchld(int signum)
             oggenc_pid = 0;
             numOggFailed++;
         }
+        else if (pid == opusenc_pid)
+        {
+            opusenc_pid = 0;
+            numOpusFailed++;
+        }
         else if (pid == flac_pid)
         {
             flac_pid = 0;
@@ -194,6 +205,11 @@ void sigchld(int signum)
             oggenc_pid = 0;
             numOggOk++;
         }
+        else if (pid == opusenc_pid)
+        {
+            opusenc_pid = 0;
+            numOpusOk++;
+        }
         else if (pid == flac_pid)
         {
             flac_pid = 0;
@@ -307,12 +323,12 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
     int start;
     int end;
     int code;
-    char type[20];
+    char type[200];
     int sector;
     
-    char trackstring[3];
+    char trackstring[4];
     
-    snprintf(trackstring, 3, "%d", tracknum);
+    snprintf(trackstring, 4, "%d", tracknum);
     
     const char * args[] = { "cdparanoia", "-e", "-d", cdrom, trackstring, filename, NULL };
     
@@ -324,6 +340,7 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
     do
     {
         pos = -1;
+        bool interrupted = FALSE;
         do
         {
             pos++;
@@ -333,10 +350,10 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
             /* signal interrupted read(), try again */
             {
                 pos--;
-                size = 1;
+                interrupted = TRUE;
             }
             
-        } while ((buf[pos] != '\n') && (size > 0) && (pos < 256));
+        } while ((size > 0 && pos < 255 && buf[pos] != '\n') || interrupted);
         buf[pos] = '\0';
 
         if ((buf[0] == 'R') && (buf[1] == 'i'))
@@ -396,13 +413,13 @@ void lame(int tracknum,
     int sector;
     int end;
     
-    char tracknum_text[3];
+    char tracknum_text[4];
     char bitrate_text[4];
     const char * args[19];
 
 //    fprintf( stderr, " lame()   Genre: %s Artist: %s Title: %s\n", genre, artist, title );	// lnr
 
-    snprintf(tracknum_text, 3, "%d", tracknum);
+    snprintf(tracknum_text, 4, "%d", tracknum);
     
     pos = 0;
     args[pos++] = "lame";
@@ -462,6 +479,7 @@ void lame(int tracknum,
     do
     {
         pos = -1;
+        bool interrupted = FALSE;
         do
         {
             pos++;
@@ -470,11 +488,11 @@ void lame(int tracknum,
             if (size == -1 && errno == EINTR)
             /* signal interrupted read(), try again */
             {
-                pos--;
                 size = 1;
+                interrupted = TRUE;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
+        } while ((size > 0 && pos < 255 && buf[pos] != '\r' && buf[pos] != '\n') || interrupted);
         buf[pos] = '\0';
         
         if (sscanf(buf, "%d/%d", &sector, &end) == 2)
@@ -523,11 +541,11 @@ void oggenc(int tracknum,
     int sector;
     int end;
 
-    char tracknum_text[3];
+    char tracknum_text[4];
     char quality_level_text[3];
     const char * args[19];
 
-    snprintf(tracknum_text, 3, "%d", tracknum);
+    snprintf(tracknum_text, 4, "%d", tracknum);
     snprintf(quality_level_text, 3, "%d", quality_level);
     
     pos = 0;
@@ -575,6 +593,7 @@ void oggenc(int tracknum,
     do
     {
         pos = -1;
+        bool interrupted = FALSE;
         do
         {
             pos++;
@@ -584,10 +603,10 @@ void oggenc(int tracknum,
             /* signal interrupted read(), try again */
             {
                 pos--;
-                size = 1;
+                interrupted = TRUE;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
+        } while ((size > 0 && pos < 255 && buf[pos] != '\r' && buf[pos] != '\n') || interrupted);
         buf[pos] = '\0';
 
         if (sscanf(buf, "\t[\t%d.%d%%]", &sector, &end) == 2)
@@ -609,6 +628,108 @@ void oggenc(int tracknum,
     }
 }
 
+// uses opusenc to encode a WAV file into a OGG and tag it
+//
+// tracknum - the track number
+// artist - the artist's name
+// album - the album the song came from
+// title - the name of the song
+// wavfilename - the path to the WAV file to encode
+// opusfilename - the path to the output OPUS file
+// bitrate - the bitrate to encode at
+// progress - the percent done
+void opusenc(int tracknum,
+            char * artist,
+            char * album,
+            char * title,
+            char * year,
+            char * genre,
+            char * wavfilename,
+            char * opusfilename,
+            int bitrate,
+            double * progress)
+{
+    int fd;
+
+    int pos;
+    char bitrate_text[4];
+    char tracknum_text[16];
+    char album_text[128];
+    char year_text[32];
+    char genre_text[64];
+    const char * args[19];
+
+    snprintf(bitrate_text, 4, "%d", int_to_bitrate(bitrate,FALSE));
+
+    pos = 0;
+    args[pos++] = "opusenc";
+    args[pos++] = "--bitrate";
+    args[pos++] = bitrate_text;
+
+    if ((tracknum > 0) && (tracknum < 100))
+    {
+        snprintf(tracknum_text,16,"TRACKNUMBER=%d",tracknum);
+        args[pos++] = "--comment";
+        args[pos++] = tracknum_text;
+    }
+    if ((artist != NULL) && (strlen(artist) > 0))
+    {
+        args[pos++] = "--artist";
+        args[pos++] = artist;
+    }
+    if ((album != NULL) && (strlen(album) > 0))
+    {
+        snprintf(album_text,128,"ALBUM=%s",album);
+        args[pos++] = "--comment";
+        args[pos++] = album_text;
+    }
+    if ((title != NULL) && (strlen(title) > 0))
+    {
+        args[pos++] = "--title";
+        args[pos++] = title;
+    }
+    if ((year != NULL) && (strlen(year) > 0))
+    {
+        snprintf(year_text,32,"DATE=%s",year);
+        args[pos++] = "--comment";
+        args[pos++] = year_text;
+    }
+    if ((genre != NULL) && (strlen(genre) > 0))
+    {
+        snprintf(genre_text,64,"GENRE=%s",genre);
+        args[pos++] = "--comment";
+        args[pos++] = genre_text;
+    }
+    args[pos++] = wavfilename;
+    args[pos++] = opusfilename;
+    args[pos++] = NULL;
+
+    fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid);
+
+    int opussize;
+    char opusbuf[256];
+    do
+    {
+        /* The opus encoder doesn't give me an estimate for completion
+        * or any way to estimate it myself, just the number of seconds
+        * done. So just sit in here until the program exits */
+        opussize = read(fd, &opusbuf[0], 256);
+
+        if (opussize == -1 && errno == EINTR)
+        /* signal interrupted read(), try again */
+            opussize = 1;
+    } while (opussize > 0);
+
+    close(fd);
+    /* don't go on until the signal for the previous call is handled */
+    while (oggenc_pid != 0)
+    {
+        debugLog("w12\n");
+        usleep(100000);
+    }
+}
+
+
 // uses the FLAC reference encoder to encode a WAV file into a FLAC and tag it
 //
 // tracknum - the track number
@@ -638,7 +759,7 @@ void flac(int tracknum,
     
     int sector;
     
-    char tracknum_text[19];
+    char tracknum_text[16];
     char * artist_text = NULL;
     char * album_text = NULL;
     char * title_text = NULL;
@@ -647,7 +768,7 @@ void flac(int tracknum,
     char compression_level_text[3];
     const char * args[19];
     
-    snprintf(tracknum_text, 15, "TRACKNUMBER=%d", tracknum);
+    snprintf(tracknum_text, 16, "TRACKNUMBER=%d", tracknum);
     
     if(artist != NULL)
     {
@@ -735,10 +856,13 @@ void flac(int tracknum,
     free(artist_text);
     free(album_text);
     free(title_text);
+    free(genre_text);
+    free(year_text);
     
     do
     {
         pos = -1;
+        bool interrupted = FALSE;
         do
         {
             pos++;
@@ -748,10 +872,10 @@ void flac(int tracknum,
             /* signal interrupted read(), try again */
             {
                 pos--;
-                size = 1;
+                interrupted = TRUE;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
+        } while ((size > 0 && pos < 255 && buf[pos] != '\r' && buf[pos] != '\n') || interrupted);
         buf[pos] = '\0';
 
         for (; pos>0; pos--)
@@ -786,7 +910,7 @@ void wavpack(int tracknum,
              int bitrate,
              double* progress)
 {
-    const char* args[8];
+    const char* args[10];
     int fd;
     int pos;
     int size;
@@ -823,6 +947,7 @@ void wavpack(int tracknum,
     do
     {
         pos = -1;
+        bool interrupted = FALSE;
         do
         {
             pos++;
@@ -832,9 +957,9 @@ void wavpack(int tracknum,
             /* signal interrupted read(), try again */
             {
                 pos--;
-                size = 1;
+                interrupted = TRUE;
             }
-        } while ((buf[pos] != '\b') && (size > 0) && (pos < 256));
+        } while ((size > 0 && pos < 255 && buf[pos] != '\b') || interrupted);
         
         buf[pos] = '\0';
         
@@ -942,8 +1067,8 @@ void musepack(char* wavfilename,
     args[pos++] = "--overwrite";
     
     args[pos++] = "--quality";
-    char qualityParam[5];
-    snprintf(qualityParam, 5, "%d.00", quality);
+    char qualityParam[6];
+    snprintf(qualityParam, 6, "%d.00", quality);
     args[pos++] = qualityParam;
     
     args[pos++] = wavfilename;
@@ -957,6 +1082,7 @@ void musepack(char* wavfilename,
     do
     {
         pos = -1;
+        bool interrupted = FALSE;
         do
         {
             pos++;
@@ -966,10 +1092,10 @@ void musepack(char* wavfilename,
             /* signal interrupted read(), try again */
             {
                 pos--;
-                size = 1;
+                interrupted = TRUE;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
+        } while ((size > 0 && pos < 256 && buf[pos] != '\r' && buf[pos] != '\n') || interrupted);
         buf[pos] = '\0';
         
         double percent;
@@ -988,14 +1114,22 @@ void musepack(char* wavfilename,
     }
 }
 
-void aac(char* wavfilename,
+void aac(int tracknum,
+         char * artist,
+         char * album,
+         char * title,
+         char * genre,
+         char * year,
+         char* wavfilename,
          char* aacfilename,
          int quality,
          double* progress)
 {
-    const char* args[8];
+    const char* args[9];
+    char* dynamic_args[6];
     int fd;
     int pos;
+    int dyn_pos;
     
     pos = 0;
     args[pos++] = "neroAacEnc";
@@ -1034,4 +1168,64 @@ void aac(char* wavfilename,
         debugLog("w11 (%d)\n", aac_pid);
         usleep(100000);
     }
+
+    /* Now add tags to the encoded track */
+
+    pos = 0;
+    args[pos++] = "neroAacTag";
+
+    args[pos++] = aacfilename;
+
+    dyn_pos = 0;
+    if (asprintf(&dynamic_args[dyn_pos], "-meta:artist=%s", artist) > 0)
+    {
+        args[pos++] = dynamic_args[dyn_pos++];
+    }
+    if (asprintf(&dynamic_args[dyn_pos], "-meta:title=%s", title) > 0)
+    {
+        args[pos++] = dynamic_args[dyn_pos++];
+    }
+    if (asprintf(&dynamic_args[dyn_pos], "-meta:album=%s", album) > 0)
+    {
+        args[pos++] = dynamic_args[dyn_pos++];
+    }
+    if (asprintf(&dynamic_args[dyn_pos], "-meta:year=%s", year) > 0)
+    {
+        args[pos++] = dynamic_args[dyn_pos++];
+    }
+    if (asprintf(&dynamic_args[dyn_pos], "-meta:genre=%s", genre) > 0)
+    {
+        args[pos++] = dynamic_args[dyn_pos++];
+    }
+    if (asprintf(&dynamic_args[dyn_pos], "-meta:track=%d", tracknum) > 0)
+    {
+        args[pos++] = dynamic_args[dyn_pos++];
+    }
+
+    args[pos++] = NULL;
+    
+    fd = exec_with_output(args, STDERR_FILENO, &aac_pid);
+    
+    do
+    {
+        /* The Nero tag writer doesn't take very long to run. Just slurp the output. */
+        size = read(fd, &buf[0], 256);
+        
+        if (size == -1 && errno == EINTR)
+        /* signal interrupted read(), try again */
+            size = 1;
+    } while (size > 0);
+    
+    close(fd);
+    /* don't go on until the signal for the previous call is handled */
+    while (aac_pid != 0)
+    {
+        debugLog("w12 (%d)\n", aac_pid);
+        usleep(100000);
+    }
+
+    while(dyn_pos)
+    {
+        free(dynamic_args[--dyn_pos]);
+    }
 }
